#!/usr/bin/env python

from os.path import abspath
import sys
import time

from .views import Screen
from .fs import Scanner


class Ohmu(object):

    refresh_rate = 0.05

    def __init__(self, root_path):
        self.screen = Screen()
        self.scanner = Scanner(root_path)
        self.keep_running = True

    def start(self):
        try:
            self.scanner.start()
            self.screen.start()
            self.loop()
        except KeyboardInterrupt:
            pass
        except:
            self.screen.stop()
            raise
        self.screen.stop()

    def loop(self):
        start = time.time()
        last_tick = start
        self.screen.tick(start, self.scanner)
        while self.keep_running:
            self.process_input(self.screen.get_key_sequence())
            if not self.keep_running:
                break
            now = time.time()
            passed = now - last_tick
            if passed > self.refresh_rate:
                last_tick = now
                self.screen.tick(now, self.scanner)

    def process_input(self, key_sequence):
        if key_sequence in (ord('q'), Screen.ESC_KEY):
            self.keep_running = False


def main(name, args):
    if name != '__main__':
        return
    root_path = abspath(args[0] if args else '.')
    Ohmu(root_path).start()


def entry_point():
    main('__main__', sys.argv[1:])


main(__name__, sys.argv[1:])



from .utils import TestCase, coffee_string


class Utils(TestCase):

    def test_coffee_string(self):
        list = [
            (
                """
                    a
                """,
                'a',
            ),
            (
                """
                    aa
                    bbb
                """,
                'aa\nbbb',
            ),
            (
                """
                    xx
                        yyyy
                        zzzz
                    nn
                """,
                'xx\n    yyyy\n    zzzz\nnn',
            ),
        ]

        for coffee, normal in list:
            self.assertEqual(coffee_string(coffee), normal)

    def test_assert_equal(self):
        string = '\n'.join([
            'Strings are different:',
            '===================================================',
            'good',
            '---------------------------------------------------',
            'bad',
            '===================================================',
        ])
        with self.assertRaisesRegexp(AssertionError, string):
            self.assertEqual('good', 'bad')



from mock import Mock, patch

from . import fs, views
from .utils import TestCase, coffee_string


class Canvas(TestCase):
    def test_limits_1_1(self):
        self.with_size(1, 1, """
            *
        """)

    def test_limits_2_1(self):
        self.with_size(2, 1, """
            <>
        """)

    def test_limits_3_1(self):
        self.with_size(3, 1, """
            <a>
        """)

    def test_limits_4_1(self):
        self.with_size(4, 1, """
            <a->
        """)

    def test_limits_5_1(self):
        self.with_size(5, 1, """
            <a-->
        """)

    def test_limits_1_2(self):
        self.with_size(1, 2, """
            ^
            v
        """)

    def test_limits_1_3(self):
        self.with_size(1, 3, """
            ^
            a
            v
        """)

    def test_limits_1_4(self):
        self.with_size(1, 4, """
            ^
            a
            |
            v
        """)

    def test_limits_1_5(self):
        self.with_size(1, 5, """
            ^
            a
            |
            |
            v
        """)

    def test_limits_2_2(self):
        self.with_size(2, 2, r"""
            /\
            \/
        """)

    def test_limits_3_2(self):
        self.with_size(3, 2, r"""
            /a\
            \-/
        """)

    def test_limits_2_3(self):
        self.with_size(2, 3, r"""
            /\
            a|
            \/
        """)

    def test_one_file(self):
        self.with_nested(10, 5, {
            'a': {'b': 5}
        }, r"""
            /a-------\
            |/b-----\|
            ||      ||
            |\------/|
            \--------/
        """
        )

    def test_small_2_2(self):
        self.with_nested(4, 4, {
            'a': {
                'b': 5,
                'c': 5,
                'd': 5,
                'e': 5,
            }
        }, r"""
            /a-\
            |**|
            |**|
            \--/
        """
        )

    def test_zero_sized_files(self):
        self.with_nested(4, 3, {
            'a': {
                'b': 0,
                'c': 0,
            }
        }, r"""
            /a-\
            |**|
            \--/
        """
        )

    def test_two_files(self):
        self.with_nested(12, 5, {
            'a': {'bb': 5, 'c': 5}
        }, r"""
            /a---------\
            |/bb-\/c--\|
            ||   ||   ||
            |\---/\---/|
            \----------/
        """
        )

    def test_complex_files(self):
        self.with_nested(42, 12, {
            'a': {
                'bb': 5,
                'c': 5,
                'dd': {
                    'f': 4,
                    'g': 8,
                },
                'e': {
                    'h': 5,
                    'i': 6,
                    'j': 8,
                    'k': {
                        'l': 14,
                    },
                },
            },
        }, r"""
            /a---------------------------------------\
            |/e---------------------\/dd-----\/bb---\|
            ||/k-------\/j--\/i----\||/g----\||     ||
            |||/l-----\||   ||     ||||     |||     ||
            ||||      |||   ||     ||||     |||     ||
            ||||      |||   ||     ||||     ||\-----/|
            ||||      |||   |\-----/|||     ||/c----\|
            ||||      |||   |/h----\||\-----/||     ||
            |||\------/||   ||     |||/f----\||     ||
            ||\--------/\---/\-----/||\-----/||     ||
            |\----------------------/\-------/\-----/|
            \----------------------------------------/
        """
        )

    def test_split_in_two_one_element(self):
        with self.assertRaises(AssertionError):
            views.Canvas.split_in_two([1])

    def test_split_in_two_two_elements(self):
        self.check_split(
            [('a', 2), ('b', 2)],
            [['a'], ['b']],
        )

    def test_split_in_two_three_elements(self):
        self.check_split(
            [('a', 3), ('b', 2), ('c', 1)],
            [['a'], ['b', 'c']],
        )

    def test_split_in_two_first_list_is_bigger(self):
        self.check_split(
            [('a', 2), ('b', 2), ('c', 1)],
            [['a', 'b'], ['c']],
        )

    def test_split_in_two_equal_sizes(self):
        self.check_split(
            [('a', 1), ('b', 1), ('c', 1), ('d', 1), ('e', 1)],
            [['a', 'b', 'c'], ['d', 'e']],
        )

    def check_split(self, files, split_names):
        files = [
            fs.File(name, size=size, path='/')
            for name, size in files
        ]
        split = views.Canvas.split_in_two(files)
        real_names = [[x.name for x in list] for list in split[0]]
        self.assertEqual(real_names, split_names)

    def with_size(self, width, height, str):
        canvas = views.Canvas(width, height)
        canvas.draw(fs.File('a', is_dir=True, path='/'))
        self.assertEqual(canvas.get_string(), coffee_string(str))

    def with_nested(self, width, height, structure, str):
        a = fs.File('a', is_dir=True, path='/')
        a.add_child(fs.File('b', size=5))

        self.assertEqual(len(structure), 1)
        root = fs.File(structure.keys()[0], is_dir=True, path='/')
        self.add_recursive(root, structure.values()[0])

        root.sortAll()

        canvas = views.Canvas(width, height)
        canvas.draw(root)

        self.assertEqual(
            canvas.get_string(),
            coffee_string(str),
        )

    def add_recursive(self, parent, structure):
        for key, value in structure.items():
            if isinstance(value, int):
                parent.add_child(fs.File(key, size=value))
            else:
                dir = fs.File(key, is_dir=True)
                parent.add_child(dir)
                self.add_recursive(dir, value)


class Screen(TestCase):
    @patch('ohmu.views.curses')
    def test_start(self, curses):
        s = views.Screen()
        screen = Mock()
        screen.getmaxyx = Mock(return_value=(20, 10))
        curses.initscr.return_value = screen
        s.start()
        self.assertEqual((s.width, s.height), (10, 20))

    @patch('ohmu.views.curses')
    def test_stop(self, curses):
        s = views.Screen()
        s.started = True
        s.screen = Mock()
        s.stop()
        curses.endwin.assert_called_with()

    @patch('ohmu.views.curses')
    def test_stop_is_no_op_is_not_started(self, curses):
        s = views.Screen()
        s.stop()
        self.assertEqual(0, curses.endwin.call_count)

    def test_escape_returns_escape(self):
        self.assert_screen_returns(
            [views.Screen.ESC_KEY, -1], views.Screen.ESC_KEY,
        )

    def test_alt_t_returns_alt_t(self):
        self.assert_screen_returns(
            [views.Screen.ESC_KEY, ord('t')],
            (views.Screen.ESC_KEY, ord('t')),
        )

    def test_q_returns_q(self):
        self.assert_screen_returns([ord('q')], ord('q'))

    def assert_screen_returns(self, sequence, ret):
        screen = views.Screen()
        screen.screen = Mock()
        screen.screen.getch.side_effect = sequence
        self.assertEqual(ret, screen.get_key_sequence())



from mock import Mock, patch

from . import Ohmu, entry_point, main, sys
from .utils import TestCase


class Main(TestCase):
    @patch('ohmu.Ohmu')
    def test_main_takes_a_dir_path(self, Ohmu):
        main('__main__', ['/dir1/2'])
        Ohmu.assert_called_with('/dir1/2')

    @patch('ohmu.Ohmu')
    @patch('ohmu.abspath')
    def test_main_uses_the_current_dir(self, abspath, Ohmu):
        abspath.return_value = '/the/abs/path'
        main('__main__', [])
        abspath.assert_called_with('.')
        Ohmu.assert_called_with('/the/abs/path')

    @patch('ohmu.main')
    def test_entry_point_works(self, main):
        with patch.object(sys, 'argv', ['-', '/some/dir']):
            entry_point()
        main.assert_called_with('__main__', ['/some/dir'])


class OhmuTest(TestCase):
    def test_exits_when_loop_exits(self):
        o = self.get_mocked_ohmu_instance()
        o.loop = Mock()
        o.start()
        o.screen.stop.assert_called_with()

    def test_exits_when_ctrl_c_is_pressed(self):
        o = self.get_mocked_ohmu_instance()
        o.loop = Mock(side_effect=KeyboardInterrupt)
        o.start()
        o.screen.stop.assert_called_with()

    def test_stop_on_exceptions_and_rethrow(self):
        o = self.get_mocked_ohmu_instance()
        o.loop = Mock(side_effect=Exception('Nucular Exception'))
        with self.assertRaisesRegexp(Exception, 'Nucular Exception'):
            o.start()
        o.screen.stop.assert_called_with()

    def test_loop_stops_on_q(self):
        o = self.get_mocked_ohmu_instance()
        o.process_input(ord('q'))
        self.assertFalse(o.keep_running)

    def test_loop_can_be_stopped_by_process_input(self):
        o = self.get_mocked_ohmu_instance()
        o.screen.get_key_sequence = Mock(return_value=ord('q'))
        o.loop()
        self.assertFalse(o.keep_running)
        self.assertEqual(o.screen.tick.call_count, 1)

    def get_mocked_ohmu_instance(self):
        o = Ohmu('/tmp')
        o.scanner = Mock()
        o.screen = Mock()
        return o



import os
from os.path import abspath, basename, join
from stat import S_ISDIR, S_ISREG
from threading import RLock, Thread


class File(object):

    def __init__(self, name, is_dir=False, parent=None, size=0, path=None):
        self.name = name
        self.is_dir = is_dir
        self.parent = parent
        self.size = size
        self.path = path
        self.children = []

    def add_child(self, file):
        file.parent = self
        file.path = join(self.path, file.name)
        self.children.append(file)

        parent = self
        while parent:
            parent.size += file.size
            parent = parent.parent

    def sortAll(self):
        self.children.sort(key=lambda x: (-x.size, x.name))
        for x in self.children:
            x.sortAll()

    @property
    def draw_size(self):
        """
        For drawing purposes, all files should have at least a size of 1 so
        that they are visible.
        """
        return self.size if self.size >= 1 else 1


class Scanner(Thread):

    def __init__(self, root_path):
        super(Scanner, self).__init__()
        self.lock = RLock()
        self.daemon = True
        path = abspath(root_path)
        self.root = File(basename(path), is_dir=True, path=path)
        self.exception = None

    def run(self):
        try:
            self.scan(self.root)
        except BaseException as e:
            self.exception = e
            raise

    def join(self):
        super(Scanner, self).join()
        if self.exception is not None:
            raise Exception(self.exception)

    def scan(self, parent):
        dirs = []
        with self.lock:
            try:
                dir_list = os.listdir(parent.path)
            except OSError:
                pass
            else:
                for f in dir_list:
                    path = join(parent.path, f)
                    if os.path.islink(path):
                        continue
                    try:
                        stat = os.stat(path)
                    except OSError:
                        continue
                    mode = stat.st_mode
                    if S_ISDIR(mode):
                        dir = File(f, is_dir=True)
                        parent.add_child(dir)
                        dirs.append(dir)
                    elif S_ISREG(mode):
                        parent.add_child(File(f, size=stat.st_size))
        for dir in dirs:
            self.scan(dir)



import curses
import math
import os


class Canvas(object):

    def __init__(self, width, height):
        self.width = width
        self.height = height
        self.table = [
            [[' ', 2] for y in xrange(width)]
            for i in xrange(height)
        ]

    def draw(self, file):
        self.draw_object(file, 0, 0, self.width - 1, 0, self.height - 1)

    def draw_object(self, object, l, sx, tx, sy, ty):  # noqa
        dx = tx - sx + 1
        dy = ty - sy + 1
        t = self.table
        name = object.name

        color = 1 + l % 6 if object.is_dir else 0
        for x in xrange(dx):
            for y in xrange(dy):
                t[sy + y][sx + x][1] = color

        if dx == 1 and dy == 1:
            t[sy][sx][0] = '*'
            return
        elif dx == 1:
            t[sy][sx][0] = '^'
            t[ty][sx][0] = 'v'
            if dy > 2:
                t[sy + 1][sx][0] = name[0]
                self.fill_vertical(t, sx, sy + 2, dy - 3)
            return
        elif dy == 1:
            t[sy][sx][0] = '<'
            t[sy][tx][0] = '>'
            self.fill_horizontal_name(name, t, sx + 1, sy, dx - 2)
            return

        t[sy][sx][0] = '/'
        t[sy][tx][0] = '\\'
        t[ty][sx][0] = '\\'
        t[ty][tx][0] = '/'

        if dx == 2 and dy > 2:
            t[sy + 1][sx][0] = name[0]
            self.fill_horizontal(t, sx + 1, sy, dx - 2)
            self.fill_vertical(t, sx, sy + 2, dy - 3)
        else:
            self.fill_horizontal_name(name, t, sx + 1, sy, dx - 2)
            self.fill_vertical(t, sx, sy + 1, dy - 2)

        self.fill_horizontal(t, sx + 1, ty, dx - 2)
        self.fill_vertical(t, tx, sy + 1, dy - 2)

        if dx > 2 and dy > 2 and object.children:
            self.draw_children(
                object.children, l + 1,
                sx + 1, tx - 1,
                sy + 1, ty - 1,
            )

    def fill_vertical(self, t, sx, sy, ny):
        for i in xrange(ny):
            t[sy + i][sx][0] = '|'

    def fill_horizontal(self, t, sx, sy, nx):
        for i in xrange(nx):
            t[sy][sx + i][0] = '-'

    def fill_horizontal_name(self, name, t, sx, sy, nx):
        if nx <= 0:
            return
        name = name[:nx]
        left = nx - len(name)
        if left > 0:
            name += '-' * left
        for i, c in enumerate(name):
            t[sy][sx + i][0] = c

    def draw_children(self, children, l, sx, tx, sy, ty):  # noqa
        if len(children) == 1:
            self.draw_object(children[0], l, sx, tx, sy, ty)
            return

        lists, sizes = self.split_in_two(children)

        dx = tx - sx + 1
        dy = ty - sy + 1
        ratio = sizes[0] / float(sizes[0] + sizes[1])
        if dx > dy:
            dx2 = int(math.ceil(dx * ratio))
            self.draw_children(
                lists[0], l,
                sx, sx + dx2 - 1,
                sy, ty,
            )
            if dx2 < dx:
                self.draw_children(
                    lists[1], l,
                    sx + dx2, tx,
                    sy, ty,
                )
        else:
            dy2 = int(math.ceil(dy * ratio))
            self.draw_children(
                lists[0], l,
                sx, tx,
                sy, sy + dy2 - 1,
            )
            if dy2 < dy:
                self.draw_children(
                    lists[1], l,
                    sx, tx,
                    sy + dy2, ty,
                )

    def get_string(self):
        return '\n'.join(''.join(y[0] for y in x) for x in self.table)

    @classmethod
    def split_in_two(cls, files):
        assert len(files) >= 2

        list_l, list_r = [files[0]], [files[-1]]
        size_l, size_r = files[0].draw_size, files[-1].draw_size
        index_l, index_r = 1, len(files) - 2

        while index_l <= index_r:
            if size_l < size_r:
                list_l.append(files[index_l])
                size_l += files[index_l].draw_size
                index_l += 1
            else:
                list_r.append(files[index_r])
                size_r += files[index_r].draw_size
                index_r -= 1

        # Make sure the first list is always the largest.
        if size_l < size_r:
            # Move one element.
            file = list_r.pop()
            size_r -= file.draw_size
            list_l.append(file)
            size_l += file.draw_size

        # Reverse since it's in the wrong order.
        list_r = list(reversed(list_r))

        return [list_l, list_r], [size_l, size_r]


class Screen(object):

    ESC_KEY = 27

    def __init__(self):
        self.height = -1
        self.width = -1
        self.started = False
        self.screen = None

        # Use a short delay for sending the Escape key (but don't override it
        # if it's set).
        os.environ.setdefault('ESCDELAY', '10')

    def start(self):
        self.screen = curses.initscr()
        curses.start_color()
        curses.use_default_colors()
        for i, color in enumerate([0, 1, 2, 3, 6, 4, 5]):
            curses.init_pair(i + 1, 15, color)

        self.started = True
        curses.noecho()
        curses.cbreak()
        curses.curs_set(False)
        self.screen.keypad(True)
        self.screen.nodelay(True)
        self.update_size()

    def tick(self, tick, scanner):
        canvas = Canvas(self.width, self.height)
        with scanner.lock:
            scanner.root.sortAll()
            canvas.draw(scanner.root)

        for i, line in enumerate(canvas.table):
            for j, [char, color] in enumerate(line):
                self.screen.insch(i, j, char, curses.color_pair(1 + color))

        self.screen.refresh()

    def update_size(self):
        self.height, self.width = self.screen.getmaxyx()

    def get_key_sequence(self):
        """Returns a key, or key sequence."""
        key = self.screen.getch()
        if key == self.ESC_KEY:  # Escape or Alt.
            self.screen.nodelay(True)
            key2 = self.screen.getch()
            if key2 == -1:
                return key  # Escape.
            return (key, key2)  # Alt.
        return key

    def stop(self):
        if not self.started:
            return
        self.screen.keypad(0)
        curses.curs_set(True)
        curses.nocbreak()
        curses.echo()
        curses.endwin()



import re
from unittest import TestCase as BaseTestCase


class TestCase(BaseTestCase):

    def equalities(self, *args):
        for a, b in zip(*(iter(args),) * 2):
            self.assertEqual(a, b)

    def assertEqual(self, a, b):
        if not (isinstance(a, basestring) or isinstance(b, basestring)):
            return super(TestCase, self).assertEqual(a, b)
        if a != b:
            raise AssertionError(coffee_string("""
                Strings are different:
                ===================================================
                %s
                ---------------------------------------------------
                %s
                ===================================================
            """) % (a, b))


def coffee_string(string):
    """
    Cuts the useless whitespaces out of a Python multiline string.

    That means this::

        print coffee_string('''
            def x(a):
                pass
        ''')

    ... is equivalent to this::

        print "def x(a):\\n    pass"
    """

    lines = string.split('\n')[1:-1]

    def start_spaces(x):
        return len(re.match(r'^ *', x).group(0))

    # Don't count empty lines.
    count_lines = (x for x in lines if len(x) > 0)
    min_spaces = min(map(start_spaces, count_lines))

    regex = re.compile(r'^ {,%s}' % min_spaces)
    return '\n'.join(map(lambda x: regex.sub('', x), lines))



import os
import shutil
from contextlib import contextmanager
from os.path import join
from tempfile import mkdtemp

from mock import Mock, patch

from . import fs
from .utils import TestCase


class File(TestCase):

    def test_dirs_have_the_size_of_their_children(self):
        a = fs.File('a', is_dir=True, path='/')
        b = fs.File('b', is_dir=True, path='/')
        c = fs.File('c', is_dir=True, path='/')
        a.add_child(b)
        b.add_child(c)

        c.add_child(fs.File('d', size=1))
        c.add_child(fs.File('e', size=1))
        b.add_child(fs.File('f', size=2))
        a.add_child(fs.File('e', size=3))

        a.sortAll()
        self.assertEqual(c.size, 2)
        self.assertEqual(b.size, 4)
        self.assertEqual(a.size, 7)

    def test_files_are_in_order(self):
        root = fs.File('p', is_dir=True, path='/')
        root.add_child(fs.File('a', size=2))
        root.add_child(fs.File('b', size=1))
        root.add_child(fs.File('c', size=3))
        root.sortAll()
        self.assertEqual([x.name for x in root.children], ['c', 'a', 'b'])


class Scanner(TestCase):

    def test_read_file_sizes(self):
        structure = {
            'd1': {
                'a': '-' * 10,
                'b': '-' * 5,
                'd2': {
                    'c': '-' * 1,
                    'd': '-' * 3,
                },
            },
        }
        with self.file_structure(structure) as dir:
            scanner = self.get_scan(join(dir, 'd1'))

        root = scanner.root
        self.equalities(
            root.name, 'd1',
            root.is_dir, True,
            root.parent, None,
            root.size, 19,
        )
        self.equalities(
            [(x.name, x.size) for x in root.children],
            [('a', 10), ('b', 5), ('d2', 4)],
        )

    def test_fobidden_files_are_ignores(self):
        structure = {
            'd1': {
                'a': '-' * 10,
                'd2': {
                    'b': '-' * 5,
                },
            },
        }

        def post_creation(dir):
            os.chmod(join(dir, 'd1', 'd2'), 0)

        def pre_deletion(dir):
            os.chmod(join(dir, 'd1', 'd2'), 0o777)

        with self.file_structure(structure, post_creation, pre_deletion) as d:
            scanner = self.get_scan(join(d, 'd1'))

        self.equalities(
            [x.name for x in scanner.root.children], ['a', 'd2'],
            scanner.root.size, 10,
            scanner.root.children[1].size, 0,
        )

    def test_deleted_files_are_ignored(self):
        structure = {
            'd1': {
                'a': '-' * 10,
                'b': '-' * 10,
            },
        }

        with self.file_structure(structure) as d:
            real_listdir = os.listdir

            with patch.object(fs.os, 'listdir') as listdir:

                def listdir_func(*args, **kwargs):
                    ret = real_listdir(*args, **kwargs)
                    os.remove(join(d, 'd1', 'b'))
                    return ret

                listdir.side_effect = listdir_func

                scanner = self.get_scan(join(d, 'd1'))

        self.equalities(
            [x.name for x in scanner.root.children], ['a'],
            scanner.root.size, 10,
        )

    def test_symlinks_are_ignored(self):
        structure = {'d1': {'a': '-' * 10}}

        def post_creation(dir):
            os.symlink(dir + '/d1/a', dir + '/d1/b')

        with self.file_structure(structure, post_creation) as d:
            scanner = self.get_scan(join(d, 'd1'))

        self.equalities(
            [x.name for x in scanner.root.children], ['a'],
            scanner.root.size, 10,
        )

    def test_scanning_error_is_raised_on_join(self):
        scanner = fs.Scanner('/bad/dir')
        scanner.scan = Mock(side_effect=Exception('The fuck is that'))
        scanner.start()
        with self.assertRaisesRegexp(Exception, 'The fuck is that'):
            scanner.join()

    @contextmanager
    def file_structure(self, structure, post_creation=None, pre_deletion=None):
        dir = mkdtemp()
        self.create_file_structure(structure, dir)
        if post_creation is not None:
            post_creation(dir)
        yield dir
        if pre_deletion is not None:
            pre_deletion(dir)
        shutil.rmtree(dir)

    def get_scan(self, dir):
        scanner = fs.Scanner(dir)
        scanner.start()
        scanner.join()
        scanner.root.sortAll()
        return scanner

    def create_file_structure(self, structure, dir):
        for name, value in structure.items():
            path = join(dir, name)
            if isinstance(value, basestring):
                open(path, 'w').write(value)
            else:
                os.mkdir(path)
                self.create_file_structure(value, path)



from setuptools import setup, find_packages

setup(
    name='ohmu',
    version='0.3.1',
    author='Paul Nechifor',
    author_email='paul@nechifor.net',
    description='View space usage in your terminal.',
    packages=find_packages(),
    keywords='space usage',
    long_description=open('readme.rst').read(),
    entry_points={'console_scripts': ['ohmu=ohmu:entry_point']},
    license='MIT',
    url='http://github.com/paul-nechifor/ohmu',
    test_suite='nose.collector',
    tests_require=['nose'],
)
